@@ -852,6 +852,8 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			pairfree(realm_check);
 			result = -1;
 		}
+
+	// User
 	} else {
 		/*
 		 *	Get the user from the database
@@ -862,18 +864,18 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 				namepair->strvalue, auth_name(authreq, 1));
 			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
 				NULL, NULL, activefd);
+			authfree(authreq);
 			pairfree(user_check);
 			pairfree(user_reply);
-			authfree(authreq);
 			return -1;
 		}
 
 		if (paircmp(authreq->request, user_check) != 0) {
 			log(L_AUTH, "Check list does not match request list [%s] (%s)", namepair->strvalue, auth_name(authreq, 1));
 			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, NULL, NULL, activefd);
+			authfree(authreq);
 			pairfree(user_check);
 			pairfree(user_reply);
-			authfree(authreq);
 			return -1;
 		}
 			
@@ -885,11 +887,29 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			if ((result = check_expiration(user_check, umsg, &user_msg))<0) {
 					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd); 
 					log(L_AUTH, "Account Expired: [%s] (%s)", namepair->strvalue, auth_name(authreq, 1));
+					authfree(authreq);
 					pairfree(user_check);
 					pairfree(user_reply);
-					authfree(authreq);
 					return -1;
 			}
+			if (result >= 0 && (check_item = pairfind(user_check, PW_ACTIVATION)) != NULL) {
+
+				time_t		now;
+				struct tm	*actdate;
+
+				now = time(0);
+
+				if (now < check_item->lvalue) {
+					actdate = localtime(&check_item->lvalue);
+					sprintf(umsg, "\r\nYour account is not yet activated\r\n\n");
+					user_msg = umsg;
+					result = -1;
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
+					log(L_ERR, "Account not activated until %d-%d-%d: [%s] (%s)", actdate->tm_year+1900, 
+						actdate->tm_mon+1, actdate->tm_mday, namepair->strvalue, auth_name(authreq, 1));
+				}
+			}
+
 
 			result = rad_check_password(authreq, activefd, user_check, namepair, pw_digest, &user_msg, userpass , dbpass);
 
@@ -1056,23 +1076,6 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 	}
 
 
-	if (result >= 0 && (check_item = pairfind(user_check, PW_ACTIVATION)) != NULL) {
-
-		time_t		now;
-		struct tm	*actdate;
-
-		now = time(0);
-
-		if (now < check_item->lvalue) {
-			actdate = localtime(&check_item->lvalue);
-			sprintf(umsg, "\r\nYour account is not yet activated\r\n\n");
-			user_msg = umsg;
-			result = -1;
-			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, user_reply, user_msg, activefd);
-			log(L_ERR, "Account not activated until %d-%d-%d: [%s] (%s)", actdate->tm_year+1900, actdate->tm_mon+1, actdate->tm_mday, namepair->strvalue, auth_name(authreq, 1));
-		}
-	}
-
 	if (result >= 0 &&
 	   (check_item = pairfind(user_check, PW_SIMULTANEOUS_USE)) != NULL) {
 #if 0 /* DEBUG */
