@@ -354,6 +354,7 @@ int sql_release_socket(SQLSOCK *socket) {
 int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
 
 	char		querystr[2048];
+	char		fixeduser[AUTH_STRING_LEN*2+1];
 	FILE		*mysqlfile;
 	int		num = 0;
 #ifdef NT_DOMAIN_HACK
@@ -387,6 +388,10 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
 	}
 #endif /* NT_DOMAIN_HACK */
 
+	// keep garbled usernames from messing up our database queries
+	sql_escape_string(fixeduser, sqlrecord->UserName, strlen(sqlrecord->UserName));
+	strNcpy(sqlrecord->UserName, fixeduser, SQLBIGREC);
+
      if (sqlrecord->AcctStatusTypeId == PW_STATUS_ACCOUNTING_ON || sqlrecord->AcctStatusTypeId == PW_STATUS_ACCOUNTING_OFF) {
         log(L_INFO, "Portmaster %s rebooted at %s", sqlrecord->NASIPAddress, sqlrecord->AcctTimeStamp);
   
@@ -468,7 +473,12 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
            /* Got stop record */
            } else {
 
-             sprintf(querystr, "SELECT RadAcctId FROM %s WHERE AcctSessionId='%s' AND NASIPAddress='%s' AND UserName='%s'", sql->config->sql_acct_table, sqlrecord->AcctSessionId, sqlrecord->NASIPAddress, sqlrecord->UserName);
+		char	authstr[AUTH_STRING_LEN+30];
+		if (sql->config->sensitiveusername) 
+			sprintf(authstr, "STRCMP(Username, '%s') = 0", sqlrecord->UserName);
+		else
+			sprintf(authstr, "UserName = '%s'", sqlrecord->UserName);
+             sprintf(querystr, "SELECT RadAcctId FROM %s WHERE AcctSessionId='%s' AND NASIPAddress='%s' AND %s", sql->config->sql_acct_table, sqlrecord->AcctSessionId, sqlrecord->NASIPAddress, authstr);
               sql_select_query(socket, querystr);
               num = sql_num_rows(socket);
 	      sql_finish_select_query(socket);
@@ -556,7 +566,7 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
  *************************************************************************/
 int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
 
-	int x;
+	int		x, ufound, gfound;
 	char		*s;
 	DICT_ATTR	*attr = NULL;
 	DICT_VALUE	*dval;
@@ -566,16 +576,28 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
 
 
 	if((attr = dict_attrfind(row[2])) == (DICT_ATTR *)NULL) {
-#if 1 /* Be quiet. */
-		log(L_ERR|L_CONS, "unknown attribute %s", row[2]);
-#endif	
+		log(L_ERR|L_CONS, "unknown attribute %s for user %s", row[2], row[1]);
 		return(-1);
 	}                              
 
 	/* If attribute is already there, skip it because we checked usercheck first 
 	   and we want user settings to over ride group settings */
-	if ((check = pairfind(*first_pair, attr->value)) != NULL && mode == PW_VP_GROUPDATA)
-		return 0;
+	check = *first_pair;
+	ufound = 0;
+	gfound = 0;
+	while (check) {
+		if (attr->value == check->attribute) {
+			if (check->source == PW_VP_USERDATA || mode == PW_VP_USERDATA)
+				ufound = 1;
+			if (check->source == PW_VP_GROUPDATA || mode == PW_VP_GROUPDATA)
+				gfound = 1;
+		}
+		
+		if (ufound && gfound)
+			return 0;
+
+		check = check->next;
+	}
 
 	if((pair = (VALUE_PAIR *)malloc(sizeof(VALUE_PAIR))) == (VALUE_PAIR *)NULL) {
 		log(L_CONS|L_ERR, "mysql_userparse: no memory");
@@ -585,6 +607,7 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
 	pair->attribute = attr->value;
 	pair->type = attr->type;
 	pair->operator = PW_OPERATOR_EQUAL;
+	pair->source = mode;
 	switch(pair->type) {
 
 		case PW_TYPE_STRING:
@@ -608,7 +631,7 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
                                    }
                         }
                         if (isdigit(*row[3])) {
-                                   pair->lvalue = atoi(row[3]);
+                                   pair->lvalue = strtol(row[3], NULL, 0);
                                    pair->length = 4;
                         }
                         else if((dval = dict_valfind(row[3])) == (DICT_VALUE *)NULL) {
@@ -672,10 +695,8 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
                         break;
 
 		default:
-                        free(pair);
-#if 1 /* Yeah yeah */
                         log(L_ERR|L_CONS, "unknown attr. type %d", pair->type);
-#endif
+                        free(pair);
                         return(-1);
 	}
 	pairadd(first_pair, pair);
@@ -720,14 +741,16 @@ int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int
 		else
 			sprintf(authstr, "%s.UserName = '%s'",sql->config->sql_usergroup_table, username);
 		sprintf(querystr, "SELECT %s.id, %s.GroupName, %s.Attribute, %s.Value FROM %s, %s WHERE %s AND %s.GroupName = %s.GroupName ORDER BY %s.id", table, table, table, table, table, sql->config->sql_usergroup_table, authstr, sql->config->sql_usergroup_table, table, table);
-	} else if (mode == PW_VP_REALMDATA)
+	} else if (mode == PW_VP_REALMDATA) {
 		sprintf(querystr, "SELECT %s.id, %s.GroupName, %s.Attribute, %s.Value FROM %s, %s WHERE %s.RealmName = '%s' AND %s.GroupName = %s.GroupName ORDER BY %s.id", table, table, table, table, table, sql->config->sql_realmgroup_table, sql->config->sql_realmgroup_table, username, sql->config->sql_realmgroup_table, table, table);
+	}
+
 	sql_select_query(socket, querystr);
 	rows = sql_num_rows(socket);
 	while ((row = sql_fetch_row(socket))) {
 
 		if (sql_userparse(vp, row, mode) != 0) {
-	 		log(L_ERR|L_CONS, "Error getting data from SQL");
+	 		log(L_ERR|L_CONS, "sql_userparse: Error adding row to pair list");
 			sql_finish_select_query(socket);
 			return -1;
 		}
@@ -764,8 +787,8 @@ static int sql_check_ts(SQL_ROW row) {
 	/*
 	 *      Find NAS type.
 	 */
-	if ((nas = nas_find(ipstr2long(row[4]))) == NULL) {
-                log(L_ERR, "Accounting: unknown NAS [%s]", row[4]);
+	if ((nas = nas_find(ipstr2long(row[3]))) == NULL) {
+                log(L_ERR, "Accounting: unknown NAS [%s]", row[3]);
                 return -1;
         }
 
@@ -818,11 +841,11 @@ static int sql_check_ts(SQL_ROW row) {
         sprintf(session_id, "%.8s", row[1]);
 
         s = CHECKRAD2;
-        execl(CHECKRAD2, "checkrad", nas->nastype, row[4], row[5],
+        execl(CHECKRAD2, "checkrad", nas->nastype, row[3], row[4],
                 row[2], session_id, nas->community, NULL);
         if (errno == ENOENT) {
                 s = CHECKRAD1;
-                execl(CHECKRAD1, "checklogin", nas->nastype, row[4], row[5],
+                execl(CHECKRAD1, "checklogin", nas->nastype, row[3], row[4],
                         row[2], session_id, nas->community, NULL);
         }
         log(L_ERR, "Check-TS: exec %s: %s", s, strerror(errno));
@@ -856,10 +879,10 @@ int sql_check_multi(SQLSOCK *socket, char *name, VALUE_PAIR *request, int maxsim
 		sprintf(authstr, "STRCMP(UserName, '%s') = 0", name);
 	else
 		sprintf(authstr, "UserName = '%s'", name);
-	sprintf(querystr, "SELECT COUNT(*) FROM %s WHERE %s AND AcctStopTime = 0", sql->config->sql_acct_table, authstr);
+	sprintf(querystr, "SELECT COUNT(*) FROM %s WHERE AcctStopTime = 0 AND %s", sql->config->sql_acct_table, authstr);
 	sql_select_query(socket, querystr);
 	row = sql_fetch_row(socket);
-	count = atoi(row[0]);
+	count = strtol(row[0], NULL, 0);
 	sql_finish_select_query(socket);
 
 	if (count < maxsimul)
@@ -872,18 +895,18 @@ int sql_check_multi(SQLSOCK *socket, char *name, VALUE_PAIR *request, int maxsim
 		ipno = htonl(fra->lvalue);
 
 	count = 0;
-	sprintf(querystr, "SELECT RadAcctId, UserName, NASIPAddress, NASPortId, FramedIPAddress FROM %s WHERE %s AND AcctStopTime = 0", sql->config->sql_acct_table, authstr);
+	sprintf(querystr, "SELECT RadAcctId, AcctSessionId, UserName, NASIPAddress, NASPortId, FramedIPAddress FROM %s WHERE AcctStopTime = 0 AND %s", sql->config->sql_acct_table, authstr);
 	sql_select_query(socket, querystr);
 	while ((row = sql_fetch_row(socket))) {
 		int check = sql_check_ts(row);
 		if (check == 1) {
 			count++;
 
-			if (ipno && atoi(row[4]) == ipno)
+			if (ipno && atoi(row[5]) == ipno)
 				mpp = 2;   
 
 		} else if (check == 2)
-			log(L_ERR,"Problem with checkrad [%s] (from nas %s)", name, row[2]);
+			log(L_ERR,"Problem with checkrad [%s] (from nas %s)", name, row[3]);
 		else {
 			/*
 			 *	False record - zap it
@@ -891,13 +914,18 @@ int sql_check_multi(SQLSOCK *socket, char *name, VALUE_PAIR *request, int maxsim
 
 			if (sql->config->deletestalesessions) {
 				SQLSOCK *sock;
+				pid_t	chldpid;
 
-   				log(L_ERR,"Deleteing stale session [%s] (from nas %s/%s)", row[1], row[2], row[3]);
-				sock = sql_create_socket();
-				sprintf(querystr, "DELETE FROM %s WHERE RadAcctId = '%s'", sql->config->sql_acct_table, row[0]);
-				sql_query(sock, querystr);
-				sql_finish_query(sock);
-				sql_close_socket(sock);
+				if ((chldpid = fork()) == 0) {
+					sock = sql_create_socket();
+					sprintf(querystr, "DELETE FROM %s WHERE RadAcctId = '%s'", sql->config->sql_acct_table, row[0]);
+					sql_query(sock, querystr);
+					sql_finish_query(sock);
+					sql_close_socket(sock);
+					exit(1);
+				} else {
+	   				log(L_ERR,"Deleteing stale session [%s] (child pid %d) (from nas %s/%s)", row[2], chldpid, row[3], row[4]);
+				}
 			}
 		}
 	}
@@ -1000,6 +1028,7 @@ int sql_read_clients(SQLSOCK *socket) {
  */
 int sql_dict_init(SQLSOCK *socket) {
 
+	char		vendorstr[64];
 	SQL_ROW		row;
 	int		line_no;
 	DICT_ATTR	*attr;
@@ -1018,10 +1047,15 @@ int sql_dict_init(SQLSOCK *socket) {
 
 	dict_free();
 
-	sprintf(querystr, "SELECT * FROM %s", sql->config->sql_dict_table);
+	sprintf(querystr, "SELECT * FROM %s ORDER BY id", sql->config->sql_dict_table);
 	sql_select_query(socket, querystr);
 	while ((row = sql_fetch_row(socket))) {
-		line_no = strtol(row[0], NULL, 0);
+		if (row[0] == NULL)
+			line_no = 0;
+		else
+			line_no = strtol(row[0], NULL, 0);
+
+		strcpy(vendorstr, row[5]);
 
 		is_attrib = 0;
 		if (strncmp(row[1], "ATTRIBUTE", 9) == 0)
@@ -1041,13 +1075,13 @@ int sql_dict_init(SQLSOCK *socket) {
 			 *	We might need to add USR to the list of
 			 *	vendors first.
 			 */
-			if (is_nmc && row[5][0] == 0) {
+			if (is_nmc && vendorstr == 0) {
 				if (!vendor_usr_seen) {
 					if (addvendor("USR", VENDORPEC_USR) < 0)
 						return -1;
 					vendor_usr_seen = 1;
 				}
-				strcpy(row[5], "USR");
+				strcpy(vendorstr, "USR");
 			}
 #endif
 
@@ -1095,13 +1129,13 @@ int sql_dict_init(SQLSOCK *socket) {
 			}
 
 			for (v = dictionary_vendors; v; v = v->next) {
-				if (strcmp(row[5], v->vendorname) == 0)
+				if (strcmp(vendorstr, v->vendorname) == 0)
 					vendor = v->vendorcode;
 			}
-			if (row[5][0] && !vendor) {
+			if (vendorstr[0] && !vendor) {
 				log(L_ERR|L_CONS,
 			"dict_init: unknown vendor %s on line %d of dictionary",
-				row[5], line_no);
+				vendorstr, line_no);
 				sql_finish_select_query(socket);
 				return -1;
 			}
@@ -1157,7 +1191,7 @@ int sql_dict_init(SQLSOCK *socket) {
 				sql_finish_select_query(socket);
 				return(-1);
 			}
-			value = atoi(row[4]);
+			value = strtol(row[4], NULL, 0);
 
 			/* Create a new VALUE entry for the list */
 			if((dval = (DICT_VALUE *)malloc(sizeof(DICT_VALUE))) ==
@@ -1195,7 +1229,7 @@ int sql_dict_init(SQLSOCK *socket) {
 				sql_finish_select_query(socket);
 				return(-1);
 			}
-			value = atoi(row[3]);
+			value = strtol(row[3], NULL, 0);
 
 			/* Create a new VENDOR entry for the list */
 			if (addvendor(row[2], value) < 0) {
