@@ -12,39 +12,6 @@
 #include 	"radiusd.h"
 
 
-/*************************************************************************
- *
- *	Function: sql_init_socket
- *
- *	Purpose: Connect to the sql server
- *
- *************************************************************************/
-int sql_init_socket(int reload) {
-
-	int i;
-
-	/* Clear up old connections if reload */
-	if (reload)
-	        for (i = 0; i < sql->config->max_sql_socks; i++)
-			if (!sql_close_socket(sql->socks[i]))
-				log(L_CONS|L_ERR, "Could not release socket %d", i);
-	
-
-        /* Initalize our connection pool */
-        for (i = 0; i < sql->config->max_sql_socks; i++) {
-	 	if ((sql->socks[i] = sql_create_socket()) == NULL) {
-	   		log(L_CONS|L_ERR, "SQL: Failed to connect socket %d", i);
-		} else {
-	   		sql->socks[i]->id = i;
-           		sql->socks[i]->in_use = 0;
-	   		DEBUG2("SQL: Connected socket %d", i);
-		}
-	}
-           
-       return 1;
-}
-
-
 /*************************************************************************
  *
  *	Function: sql_query
@@ -61,28 +28,14 @@ SQLSOCK *sql_create_socket(void) {
 	}
 
 	mysql_init(&(socket->conn));
-	if (!(socket->sock = mysql_real_connect(&(socket->conn), sql->config->mysql_server, sql->config->mysql_login, sql->config->mysql_password, sql->config->mysql_db, 0, NULL, 0))) {
-		log(L_ERR, "Init: Couldn't connect socket to MySQL server %s@%s:%s", sql->config->mysql_login, sql->config->mysql_server, sql->config->mysql_db);
+	if (!(socket->sock = mysql_real_connect(&(socket->conn), sql->config->sql_server, sql->config->sql_login, sql->config->sql_password, sql->config->sql_db, 0, NULL, 0))) {
+		log(L_ERR, "Init: Couldn't connect socket to MySQL server %s@%s:%s", sql->config->sql_login, sql->config->sql_server, sql->config->sql_db);
 		socket->sock = NULL;
 		return NULL;
 	}
 	return socket;
 }
 
-/*************************************************************************
- *
- *	Function: sql_close_socket
- *
- *	Purpose: Close and free a sql socket
- *
- *************************************************************************/
-int sql_close_socket(SQLSOCK *socket) {
-	DEBUG2("SQL: Closing socket %d", socket->id);
-	mysql_close(socket->sock);
-	free(socket);
-	return 1;
-}
- 
 /*************************************************************************
  *
  *	Function: sql_query
@@ -92,10 +45,13 @@ int sql_close_socket(SQLSOCK *socket) {
  *************************************************************************/
 int sql_query(SQLSOCK *socket, char *querystr) {
 
- if (sql->config->sqltrace)
-	DEBUG(querystr);
- return mysql_query(socket->sock, querystr);
-
+	if (sql->config->sqltrace)
+		DEBUG(querystr);
+	 if (socket->sock == NULL) {
+		log(L_ERR, "Socket not connected");
+		return 0;
+	}
+	return mysql_query(socket->sock, querystr);
 }
 
 
@@ -108,14 +64,17 @@ int sql_query(SQLSOCK *socket, char *querystr) {
  *************************************************************************/
 int sql_select_query(SQLSOCK *socket, char *querystr) {
 
- if (sql->config->sqltrace)
-	DEBUG(querystr);
- mysql_query(socket->sock, querystr);
- if (sql_store_result(socket) && sql_num_fields(socket)) 
-	return 1;
- else
-	return 0;
-
+	if (sql->config->sqltrace)
+		DEBUG(querystr);
+	if (socket->sock == NULL) {
+		log(L_ERR, "Socket not connected");
+		return 0;
+	}
+	mysql_query(socket->sock, querystr);
+	if (sql_store_result(socket) && sql_num_fields(socket)) 
+		return 1;
+	else
+		return 0;
 }
 
 
@@ -129,10 +88,13 @@ int sql_select_query(SQLSOCK *socket, char *querystr) {
  *************************************************************************/
 int sql_store_result(SQLSOCK *socket) {
 
+	if (socket->sock == NULL) {
+		log(L_ERR, "Socket not connected");
+		return 0;
+	}
 	if (!(socket->result = mysql_store_result(socket->sock))) {
 		log(L_ERR,"MYSQL Error: Cannot get result");
 		log(L_ERR,"MYSQL Error: %s",mysql_error(socket->sock));
-		sql_close(socket);
 		return 0;
 	}
 	return 1;
@@ -158,7 +120,6 @@ int sql_num_fields(SQLSOCK *socket) {
 #endif
 		log(L_ERR,"MYSQL Error: Cannot get result");
 		log(L_ERR,"MYSQL error: %s",mysql_error(socket->sock));
-		sql_close(socket);
 	}
 	return num;
 }
@@ -174,8 +135,7 @@ int sql_num_fields(SQLSOCK *socket) {
  *************************************************************************/
 int sql_num_rows(SQLSOCK *socket) {
 
-    return mysql_num_rows(socket->result);
-
+	return mysql_num_rows(socket->result);
 }
 
 
@@ -189,8 +149,7 @@ int sql_num_rows(SQLSOCK *socket) {
  *************************************************************************/
 SQL_ROW sql_fetch_row(SQLSOCK *socket) {
 
-   return mysql_fetch_row(socket->result);
-
+	return mysql_fetch_row(socket->result);
 }
 
 
@@ -205,8 +164,7 @@ SQL_ROW sql_fetch_row(SQLSOCK *socket) {
  *************************************************************************/
 void sql_free_result(SQLSOCK *socket) {
 
-   mysql_free_result(socket->result);
-
+	mysql_free_result(socket->result);
 }
 
 
@@ -221,8 +179,7 @@ void sql_free_result(SQLSOCK *socket) {
  *************************************************************************/
 char *sql_error(SQLSOCK *socket) {
 
-  return (mysql_error(socket->sock));
-
+	return mysql_error(socket->sock);
 }
 
 
@@ -236,8 +193,8 @@ char *sql_error(SQLSOCK *socket) {
  *************************************************************************/
 void sql_close(SQLSOCK *socket) {
 
-   mysql_close(socket->sock);
-
+	mysql_close(socket->sock);
+	socket->sock = NULL;
 }
 
 
@@ -263,7 +220,7 @@ void sql_finish_query(SQLSOCK *socket) {
  *************************************************************************/
 void sql_finish_select_query(SQLSOCK *socket) {
 
-   sql_free_result(socket);
+	sql_free_result(socket);
 }
 
 
@@ -275,10 +232,8 @@ void sql_finish_select_query(SQLSOCK *socket) {
  *
  *************************************************************************/
 int sql_affected_rows(SQLSOCK *socket) {
-   int rows;
 
-   rows = mysql_affected_rows(socket->sock);
-   return rows;
+	return mysql_affected_rows(socket->sock);
 }
 
 
