@@ -10,20 +10,20 @@
 typedef MYSQL_ROW	SQL_ROW;
 
 typedef struct sql_config {
-        char mysql_server[40];
-        char mysql_login[20];
-        char mysql_password[20];
-        char mysql_db[20];
-        char mysql_acct_table[MAX_TABLE_LEN];
-        char mysql_authcheck_table[MAX_TABLE_LEN];
-        char mysql_authreply_table[MAX_TABLE_LEN];
-        char mysql_groupcheck_table[MAX_TABLE_LEN];
-        char mysql_groupreply_table[MAX_TABLE_LEN];
-        char mysql_usergroup_table[MAX_TABLE_LEN];
-        char mysql_realm_table[MAX_TABLE_LEN];
-        char mysql_realmgroup_table[MAX_TABLE_LEN];
-        char mysql_nas_table[MAX_TABLE_LEN];
-        char mysql_dict_table[MAX_TABLE_LEN];
+        char sql_server[40];
+        char sql_login[20];
+        char sql_password[20];
+        char sql_db[20];
+        char sql_acct_table[MAX_TABLE_LEN];
+        char sql_authcheck_table[MAX_TABLE_LEN];
+        char sql_authreply_table[MAX_TABLE_LEN];
+        char sql_groupcheck_table[MAX_TABLE_LEN];
+        char sql_groupreply_table[MAX_TABLE_LEN];
+        char sql_usergroup_table[MAX_TABLE_LEN];
+        char sql_realm_table[MAX_TABLE_LEN];
+        char sql_realmgroup_table[MAX_TABLE_LEN];
+        char sql_nas_table[MAX_TABLE_LEN];
+        char sql_dict_table[MAX_TABLE_LEN];
         int  sensitiveusername;
         int  sqltrace;
 	int  deletestalesessions;
@@ -36,6 +36,7 @@ typedef struct sql_socket {
         MYSQL_RES	*result;
         int             id;
         int             in_use;
+	struct timeval	tv;
 } SQLSOCK;
  
 typedef struct sql {
@@ -44,9 +45,7 @@ typedef struct sql {
 } SQL;
 
 
-int sql_init_socket(int reload);
 SQLSOCK *sql_create_socket(void);
-int sql_close_socket(SQLSOCK *socket);
 int sql_checksocket(const char *facility);
 int sql_query(SQLSOCK *socket, char *querystr);
 int sql_select_query(SQLSOCK *socket, char *querystr);
